package dk.nsp.backgroundjob.ws;

import dk.nsp.backgroundjob.health.HealthIndication;
import dk.nsp.backgroundjob.health.HealthIndicator;
import org.springframework.beans.factory.annotation.Autowired;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.stream.Collectors;

public class AbstractHealthServlet extends AbstractServlet {
	@Autowired
	private List<HealthIndicator> healthIndicators;

	@Override
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException {
		reportHealth(response);
	}

	@Override
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {
		reportHealth(response);
	}

	private void reportHealth(HttpServletResponse response) throws IOException {
		int status = 200;
		String body = "";
		if (healthIndicators != null) {
			List<HealthIndication> healthIndications = healthIndicators.stream().map(HealthIndicator::getHealthInfo).collect(Collectors.toList());
			status = healthIndications.stream().mapToInt(HealthIndication::getStatus).max().orElse(200);
			body = healthIndications.stream().map(HealthIndication::getText).collect(Collectors.joining(","));
		}

		response.getOutputStream().write(String.format("{%s}", body).getBytes());
		response.setStatus(status);
	}
}