package dk.nsp.backgroundjob.ws;

import dk.nsp.backgroundjob.health.HealthIndication;
import dk.nsp.backgroundjob.health.HealthIndicator;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mockito;
import org.mockito.Spy;
import org.mockito.runners.MockitoJUnitRunner;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.mock.web.MockHttpServletResponse;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

@RunWith(MockitoJUnitRunner.class)
public class AbstractHealthServletTest {
    @Spy
    private List<HealthIndicator> healthIndicators = new ArrayList<>();

    @InjectMocks
    private AbstractHealthServlet subject;

    @Test
    public void testEmpty() throws IOException {

        // Given
        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();

        // When
        subject.doGet(request, response);

        // Then
        Assert.assertEquals(200, response.getStatus());
        Assert.assertEquals("{}", response.getContentAsString());
    }

    @Test
    public void testWithHealthIndication() throws IOException {
        // Given
        HealthIndicator mockIndication1 = Mockito.mock(HealthIndicator.class);
        Mockito.when(mockIndication1.getHealthInfo()).thenReturn(new HealthIndication(500, "\"test1\": \"abc\""));
        HealthIndicator mockIndication2 = Mockito.mock(HealthIndicator.class);
        Mockito.when(mockIndication2.getHealthInfo()).thenReturn(new HealthIndication(200, "\"test2\": \"xyz\""));
        healthIndicators.add(mockIndication1);
        healthIndicators.add(mockIndication2);
        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();

        // When
        subject.doGet(request, response);

        // Then
        Assert.assertEquals(500, response.getStatus());
        Assert.assertEquals("{\"test1\": \"abc\",\"test2\": \"xyz\"}", response.getContentAsString());

    }
}
