package dk.nsp.backgroundjob.ws;

import dk.nsp.backgroundjob.service.StackedOperationsService;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.mock.web.MockHttpServletResponse;

import java.io.IOException;

import static org.junit.Assert.assertEquals;

public class AbstractOperationsServletTest {
    private StackedOperationsService operationsService;
    private AbstractOperationsServlet operationsServlet;

    @Before
    public void setUp() throws Exception {
        operationsService = Mockito.mock(StackedOperationsService.class);
        operationsServlet = new AbstractOperationsServlet() {
            @Override
            protected StackedOperationsService getOperationsService() {
                return operationsService;
            }

            @Override
            public String getServletName() {
                return "Test servlet";
            }
        };
    }

    @Test
    public void doGet_WhenRunSucceeds_Responds200() throws IOException {
        // Arrange
        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();

        // Act
        operationsServlet.doGet(request, response);

        // Assert
        assertEquals(200, response.getStatus());
    }

    @Test
    public void doGet_WhenRunFails_Responds500() throws IOException {
        // Arrange
        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();
        Mockito.doThrow(Exception.class).when(operationsService).run();

        // Act
        operationsServlet.doGet(request, response);

        // Assert
        assertEquals(500, response.getStatus());
    }
}