package dk.nsp.xdscleanup.common.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

public class CleanupUtils {
    // Should not be instantiated
    private CleanupUtils() {
    }

    public static <T> Collection<List<T>> createShuffledBatches(Collection<T> collection, int batchSize) {
        if (batchSize < 1) {
            throw new IllegalArgumentException("Batch size must be at least 1");
        }
        ArrayList<T> list = new ArrayList<>(collection);
        Collections.shuffle(list);
        AtomicInteger counter = new AtomicInteger();
        return list.stream()
                .collect(Collectors.groupingBy(entry -> counter.getAndIncrement() / batchSize))
                .values();
    }
}
