package dk.nsp.xdscleanup.common.util;

import org.junit.Test;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class CleanupUtilsTest {

    @Test
    public void createShuffledBatches_WithEmptyList_ReturnsEmptyList() {
        Collection<List<Object>> batches = CleanupUtils.createShuffledBatches(Collections.emptyList(), 100);

        assertTrue(batches.isEmpty());
    }

    @Test(expected = IllegalArgumentException.class)
    public void createShuffledBatches_WithBatchSize0_ThrowsException() {
        CleanupUtils.createShuffledBatches(Collections.emptyList(), 0);
    }

    @Test(expected = IllegalArgumentException.class)
    public void createShuffledBatches_WithNegativeBatchSize_ThrowsException() {
        CleanupUtils.createShuffledBatches(Collections.emptyList(), -5);
    }

    @Test
    public void createShuffledBatches_WithListOf5EntriesAndBatchSize3_Returns2BatchesWithAllEntries() {
        List<Object> inputList = Arrays.asList(1, 2, 3, 4, 5);
        Collection<List<Object>> batches = CleanupUtils.createShuffledBatches(inputList, 3);

        assertEquals(2, batches.size());
        List<Integer> batchSizes = batches.stream().map(List::size).collect(Collectors.toList());
        assertTrue(batchSizes.containsAll(Arrays.asList(2, 3)));
        List<Object> batchEntries = batches.stream().flatMap(Collection::stream).collect(Collectors.toList());
        assertEquals(5, batchEntries.size());
        assertTrue(batchEntries.containsAll(inputList));
    }
}