package dk.nsp.xdscleanup.deprecated.factory;

import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.common.model.SubmissionSet;
import dk.nsp.xdscleanup.deprecated.model.DocumentTypeDeprecatedCleanup;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import dk.nsp.xdscleanup.deprecated.stack.registry.deprecated.DeprecatedDocumentPatientIdPrefixCleanupSupplier;
import dk.nsp.xdscleanup.deprecated.stack.registry.deprecated.RegistryBatchCleanupOperation;
import dk.nsp.xdscleanup.deprecated.stack.registry.submissionsets.RegistrySubmissionSetsBatchCleanupOperation;

import java.util.List;

public class CleanupOperationsFactory {
    private final RegistryDeletionService registryDeletionService;
    private final DateTimeProvider dateTimeProvider;
    private final int registryBatchSize;
    private final int daysBeforeDelete;

    private final int deprecatedLimit;
    private final int submissionSetLimit;

    public CleanupOperationsFactory(RegistryDeletionService registryDeletionService,
                                    DateTimeProvider dateTimeProvider,
                                    int registryBatchSize,
                                    int daysBeforeDelete,
                                    int deprecatedLimit,
                                    int submissionSetLimit) {
        this.registryDeletionService = registryDeletionService;
        this.dateTimeProvider = dateTimeProvider;
        this.registryBatchSize = registryBatchSize;
        this.daysBeforeDelete = daysBeforeDelete;
        this.deprecatedLimit = deprecatedLimit;
        this.submissionSetLimit = submissionSetLimit;
    }

    public DeprecatedDocumentPatientIdPrefixCleanupSupplier createDeprecatedDocumentBatchCleanupOperation(String patientIdPrefix, DocumentTypeDeprecatedCleanup documentTypeDeprecatedCleanup) {
        return new DeprecatedDocumentPatientIdPrefixCleanupSupplier(registryDeletionService, this, registryBatchSize, patientIdPrefix, documentTypeDeprecatedCleanup, daysBeforeDelete, deprecatedLimit);
    }

    public RegistryBatchCleanupOperation createRegistryBatchCleanupOperation(List<DocumentEntry> documentEntriesForDeletion) {
        return new RegistryBatchCleanupOperation(registryDeletionService, documentEntriesForDeletion, dateTimeProvider);
    }

    public RegistrySubmissionSetsBatchCleanupOperation createRegistrySubmissionSetsBatchCleanupOperation(List<SubmissionSet> submissionSetsForDeletion) {
        return new RegistrySubmissionSetsBatchCleanupOperation(registryDeletionService, submissionSetsForDeletion, dateTimeProvider);
    }
}
