package dk.nsp.xdscleanup.deprecated.setup;

import dk.nsp.backgroundjob.service.StackedOperationsService;
import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.xdscleanup.deprecated.factory.CleanupOperationsFactory;
import dk.nsp.xdscleanup.deprecated.model.DocumentTypeDeprecatedCleanup;
import dk.nsp.xdscleanup.nxrg.dao.NxrgDao;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import dk.nsp.xdscleanup.deprecated.stack.registry.deprecated.DeprecatedDocumentCleanupSupplier;
import dk.nsp.xdscleanup.deprecated.stack.registry.submissionsets.DanglingSubmissionSetsCleanupSupplier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.PropertySource;
import org.springframework.context.support.PropertySourcesPlaceholderConfigurer;

import java.time.Duration;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@Configuration
@PropertySource("classpath:xdscleanup-deprecated.properties")
@Import({DBSetup.class, VersionSetup.class})
public class XdsCleanupSetup {

    @Value("${desired.execution.duration}")
    private String desiredExecutionDuration;

    @Value("${deletion.registry.batchsize}")
    private int batchSizeRegistry;

    @Value("${deletion.deprecated.document.cleanup.typecodes}")
    private String[] deprecatedDocumentCleanupTypeCodesStr;

    @Value("${deletion.deprecated.document.cleanup.days}")
    private String deprecatedDocumentCleanupDaysStr;

    @Value("${deletion.deprecated.document.cleanup.limit}")
    private String deprecatedDocumentCleanupLimitStr;

    @Value("${deletion.dangling.submissionsets.cleanup.limit}")
    private String danglingSubmissionsetsCleanupLimitStr;

    @Bean
    public static PropertySourcesPlaceholderConfigurer propertySourcesPlaceholderConfigurer() {
        return new PropertySourcesPlaceholderConfigurer();
    }

    @Bean
    public RegistryDeletionService registryDeletionService(NxrgDao nxrgDao) {
        return new RegistryDeletionService(nxrgDao);
    }

    @Bean
    public CleanupOperationsFactory cleanupOperationsFactory(RegistryDeletionService registryDeletionService, DateTimeProvider dateTimeProvider, int deprecatedDocumentCleanupDays, int deprecatedDocumentCleanupLimit, int danglingSubmissionsetsCleanupLimit) {
        return new CleanupOperationsFactory(registryDeletionService, dateTimeProvider, batchSizeRegistry,  deprecatedDocumentCleanupDays, deprecatedDocumentCleanupLimit, danglingSubmissionsetsCleanupLimit);
    }

    @Bean
    public StackedOperationsService registryDeprecatedDocumentCleanupService(CleanupOperationsFactory cleanupOperationsFactory, Duration desiredExecutionDuration,
                                                                             DateTimeProvider dateTimeProvider, List<DocumentTypeDeprecatedCleanup> documentTypeDeprecatedCleanupList) {
        DeprecatedDocumentCleanupSupplier defaultStackOperation = new DeprecatedDocumentCleanupSupplier(cleanupOperationsFactory, documentTypeDeprecatedCleanupList);
        return new StackedOperationsService(defaultStackOperation, desiredExecutionDuration, dateTimeProvider).withServiceName("Registry Deprecated Document Cleanup");
    }

    @Bean
    public StackedOperationsService registryDanglingSubmissionSetsCleanupService(RegistryDeletionService registryDeletionService, CleanupOperationsFactory cleanupOperationsFactory, Duration desiredExecutionDuration,
                                                                   DateTimeProvider dateTimeProvider, int danglingSubmissionsetsCleanupLimit) {
        DanglingSubmissionSetsCleanupSupplier defaultStackOperation = new DanglingSubmissionSetsCleanupSupplier(registryDeletionService, cleanupOperationsFactory, batchSizeRegistry, danglingSubmissionsetsCleanupLimit);
        return new StackedOperationsService(defaultStackOperation, desiredExecutionDuration, dateTimeProvider).withServiceName("Registry Dangling SubmissionSets Cleanup");
    }

    @Bean
    public DateTimeProvider dateTimeProvider() {
        return new DateTimeProvider();
    }

    @Bean List<DocumentTypeDeprecatedCleanup> documentTypeDeprecatedCleanupList() {
        return Arrays.stream(deprecatedDocumentCleanupTypeCodesStr)
                .map(DocumentTypeDeprecatedCleanup::parse)
                .collect(Collectors.toList());
    }

    @Bean
    public int deprecatedDocumentCleanupDays() {
        return Integer.parseInt(deprecatedDocumentCleanupDaysStr);
    }

    @Bean
    public Duration desiredExecutionDuration() {
        return Duration.parse(desiredExecutionDuration);
    }

    @Bean
    public int deprecatedDocumentCleanupLimit() {
        return Integer.parseInt(deprecatedDocumentCleanupLimitStr);
    }

    @Bean
    public int danglingSubmissionsetsCleanupLimit() {
        return Integer.parseInt(danglingSubmissionsetsCleanupLimitStr);
    }
}
