package dk.nsp.xdscleanup.deprecated.stack.registry.deprecated;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.xdscleanup.deprecated.factory.CleanupOperationsFactory;
import dk.nsp.xdscleanup.deprecated.model.DocumentTypeDeprecatedCleanup;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.IntStream;


public class DeprecatedDocumentCleanupSupplier implements StackOperation {

    private final CleanupOperationsFactory cleanupOperationsFactory;
    private final List<DocumentTypeDeprecatedCleanup> documentTypeDeprecatedCleanupList;
    private static final List<Integer> DAYS = IntStream.range(1, 32).boxed().collect(Collectors.toList());
    private static final List<Integer> MONTHS = IntStream.range(1, 13).boxed().collect(Collectors.toList());
    private static final List<String> PREFIXES = DAYS.stream()
            .flatMap(day -> MONTHS.stream().map(month -> patientIdPrefix(day, month)))
            .collect(Collectors.toList());

    public DeprecatedDocumentCleanupSupplier(CleanupOperationsFactory cleanupOperationsFactory, List<DocumentTypeDeprecatedCleanup> documentTypeDeprecatedCleanupList) {
        this.cleanupOperationsFactory = cleanupOperationsFactory;
        this.documentTypeDeprecatedCleanupList = documentTypeDeprecatedCleanupList;
    }

    @Override
    public List<StackOperation> run() {
        List<StackOperation> list = new ArrayList<>();
        documentTypeDeprecatedCleanupList.forEach(documentTypeDeprecatedCleanup -> {
            PREFIXES.stream().map(PREFIX -> cleanupOperationsFactory.createDeprecatedDocumentBatchCleanupOperation(PREFIX, documentTypeDeprecatedCleanup)).forEach(list::add);
        });
        return list;

    }

    private static String patientIdPrefix(int day, int month) {
        return StringUtils.leftPad(day + "", 2, "0") + StringUtils.leftPad(month + "", 2, "0");
    }
}
