package dk.nsp.xdscleanup.deprecated.stack.registry.deprecated;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.common.util.CleanupException;
import dk.nsp.xdscleanup.common.util.CleanupUtils;
import dk.nsp.xdscleanup.deprecated.factory.CleanupOperationsFactory;
import dk.nsp.xdscleanup.deprecated.model.DocumentTypeDeprecatedCleanup;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;

import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

public class DeprecatedDocumentPatientIdPrefixCleanupSupplier implements StackOperation {

    private final RegistryDeletionService registryDeletionService;
    private final CleanupOperationsFactory cleanupOperationsFactory;
    private final String patientIdPrefix;
    private final int batchSize;
    private final DocumentTypeDeprecatedCleanup documentTypeDeprecatedCleanup;
    private final int daysBeforeDelete;
    private final int limit;

    public DeprecatedDocumentPatientIdPrefixCleanupSupplier(RegistryDeletionService registryDeletionService, CleanupOperationsFactory cleanupOperationsFactory, int batchSize, String patientIdPrefix,
                                                            DocumentTypeDeprecatedCleanup documentTypeDeprecatedCleanup, int daysBeforeDelete, int limit) {
        this.registryDeletionService = registryDeletionService;
        this.cleanupOperationsFactory = cleanupOperationsFactory;
        this.patientIdPrefix = patientIdPrefix;
        this.batchSize = batchSize;
        this.documentTypeDeprecatedCleanup = documentTypeDeprecatedCleanup;
        this.daysBeforeDelete = daysBeforeDelete;
        this.limit = limit;
    }

    @Override
    public List<StackOperation> run() {
        List<DocumentEntry> documentEntriesForDeletion = getDeprecatedDocumentEntriesForDeletion(patientIdPrefix, documentTypeDeprecatedCleanup, daysBeforeDelete, limit);
        Collection<List<DocumentEntry>> entryBatches = CleanupUtils.createShuffledBatches(documentEntriesForDeletion, batchSize);

        return entryBatches.stream()
                .map(batch -> cleanupOperationsFactory.createRegistryBatchCleanupOperation(batch).withOperationName("Registry Deprecated Document Entries Cleanup"))
                .collect(Collectors.toList());
    }

    private List<DocumentEntry> getDeprecatedDocumentEntriesForDeletion(String patientIdPrefix, DocumentTypeDeprecatedCleanup documentTypeDeprecatedCleanup, int daysBeforeDelete, int limit) {
        try {
            return registryDeletionService.getDeprecatedDocumentsForDeletion(patientIdPrefix, documentTypeDeprecatedCleanup.getTypeCode(), daysBeforeDelete, limit);
        } catch (Exception e) {
            throw new CleanupException(String.format("Failed to retrieve deprecated document entries with type codes %s and created before %s", documentTypeDeprecatedCleanup.getTypeCode(), daysBeforeDelete), e);
        }
    }
}
