package dk.nsp.xdscleanup.deprecated.stack.registry.deprecated;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.common.util.CleanupException;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import org.apache.log4j.Logger;

import java.time.Duration;
import java.time.ZonedDateTime;
import java.util.Collections;
import java.util.List;

public class RegistryBatchCleanupOperation implements StackOperation {
    private final Logger log = Logger.getLogger(RegistryBatchCleanupOperation.class);
    private final RegistryDeletionService registryDeletionService;
    private final List<DocumentEntry> documentEntriesForDeletion;
    private final DateTimeProvider dateTimeProvider;
    private String operationName = "Registry Deprecated Document Batch Cleanup";

    public RegistryBatchCleanupOperation(RegistryDeletionService registryDeletionService,
                                         List<DocumentEntry> documentEntriesForDeletion,
                                         DateTimeProvider dateTimeProvider) {
        this.registryDeletionService = registryDeletionService;
        this.documentEntriesForDeletion = documentEntriesForDeletion;
        this.dateTimeProvider = dateTimeProvider;
    }

    @Override
    public List<StackOperation> run() {
        log.trace(String.format("%s: Starting batch deletion of %d entries.", operationName, documentEntriesForDeletion.size()));
        ZonedDateTime startTime = dateTimeProvider.now();

        try {
            registryDeletionService.deleteDocumentEntries(documentEntriesForDeletion);
        } catch (Exception e) {
            throw new CleanupException("Failed to delete document entries " + documentEntriesForDeletion, e);
        }

        log.info(String.format("%s: Batch cleanup finished! deleted=%d Time spent: %d seconds.", operationName, documentEntriesForDeletion.size(), Duration.between(startTime, dateTimeProvider.now()).getSeconds()));
        log.info(String.format("%s: Deleted DocumentEntries are: %s", operationName, appendDocumentEntries(documentEntriesForDeletion)));
        return Collections.emptyList();
    }

    public RegistryBatchCleanupOperation withOperationName(String operationName) {
        this.operationName = operationName;
        return this;
    }

    private String appendDocumentEntries(List<DocumentEntry> documentEntries) {
        StringBuilder sb = new StringBuilder();
        for (DocumentEntry documentEntry : documentEntries) {
            if (sb.length() != 0) {
                sb.append(",");
            }
            sb.append("uniqueId=" + documentEntry.getUniqueId());
        }
        return sb.toString();
    }
}
