package dk.nsp.xdscleanup.deprecated.stack.registry.submissionsets;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.xdscleanup.common.model.SubmissionSet;
import dk.nsp.xdscleanup.common.util.CleanupException;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import org.apache.log4j.Logger;

import java.time.Duration;
import java.time.ZonedDateTime;
import java.util.Collections;
import java.util.List;

public class RegistrySubmissionSetsBatchCleanupOperation implements StackOperation {

    private final Logger log = Logger.getLogger(RegistrySubmissionSetsBatchCleanupOperation.class);

    private final RegistryDeletionService registryDeletionService;
    private final List<SubmissionSet> submissionSetsForDeletion;
    private final DateTimeProvider dateTimeProvider;
    private String operationName = "Registry SubmissionSets Batch Cleanup";

    public RegistrySubmissionSetsBatchCleanupOperation(RegistryDeletionService registryDeletionService, List<SubmissionSet> submissionSetsForDeletion, DateTimeProvider dateTimeProvider) {
        this.registryDeletionService = registryDeletionService;
        this.submissionSetsForDeletion = submissionSetsForDeletion;
        this.dateTimeProvider = dateTimeProvider;
    }

    @Override
    public List<StackOperation> run() {
        log.trace(String.format("%s: Starting batch deletion of %d submissionsets.", operationName, submissionSetsForDeletion.size()));
        ZonedDateTime startTime = dateTimeProvider.now();

        try {
            registryDeletionService.deleteSubmissionSets(submissionSetsForDeletion);
        } catch (Exception e) {
            throw new CleanupException("Failed to delete submissionsets " + submissionSetsForDeletion, e);
        }

        log.info(String.format("%s: Batch cleanup finished! deleted=%d Time spent: %d seconds.", operationName, submissionSetsForDeletion.size(), Duration.between(startTime, dateTimeProvider.now()).getSeconds()));
        log.info(String.format("%s: Deleted SubmissionSets are: %s", operationName, submissionSetsForDeletion));
        return Collections.emptyList();
    }

    public RegistrySubmissionSetsBatchCleanupOperation withOperationName(String operationName) {
        this.operationName = operationName;
        return this;
    }
}
