package dk.nsp.xdscleanup.deprecated.stack.registry.deprecated;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.common.model.xds.TypeCode;
import dk.nsp.xdscleanup.common.util.CleanupException;
import dk.nsp.xdscleanup.deprecated.factory.CleanupOperationsFactory;
import dk.nsp.xdscleanup.deprecated.model.DocumentTypeDeprecatedCleanup;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Mockito;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import static dk.nsp.xdscleanup.deprecated.TestUtils.assertThrows;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class DeprecatedDocumentPatientIdPrefixCleanupSupplierTest {

    private static final int BATCH_SIZE = 2;
    private static final int DAYSBEFOREDELETE = 60;
    private static final int LIMIT = 10000;
    private DeprecatedDocumentPatientIdPrefixCleanupSupplier deprecatedDocumentPatientIdPrefixCleanupSupplier;
    private RegistryDeletionService registryDeletionService;
    private CleanupOperationsFactory cleanupOperationsFactory;

    @Before
    public void setUp() {
        registryDeletionService = Mockito.mock(RegistryDeletionService.class);
        cleanupOperationsFactory = Mockito.mock(CleanupOperationsFactory.class);
        DocumentTypeDeprecatedCleanup documentTypeDeprecatedCleanup = new DocumentTypeDeprecatedCleanup(new TypeCode("abc", "1.1.1.1"));
        deprecatedDocumentPatientIdPrefixCleanupSupplier = new DeprecatedDocumentPatientIdPrefixCleanupSupplier(registryDeletionService, cleanupOperationsFactory, BATCH_SIZE,  "0101", documentTypeDeprecatedCleanup, DAYSBEFOREDELETE, LIMIT);
    }

    @Test
    public void run_With5EntriesAndBatchSize2_Returns3BatchOperations() {

        // Arrange
        DocumentEntry documentEntry1 = new DocumentEntry(1, "1", "1");
        DocumentEntry documentEntry2 = new DocumentEntry(2, "2", "2");
        DocumentEntry documentEntry3 = new DocumentEntry(3, "3", "3");
        DocumentEntry documentEntry4 = new DocumentEntry(4, "4", "4");
        DocumentEntry documentEntry5 = new DocumentEntry(5, "5", "5");
        RegistryBatchCleanupOperation operation1 = Mockito.mock(RegistryBatchCleanupOperation.class);
        Mockito.when(operation1.withOperationName(Mockito.anyString())).thenReturn(operation1);
        RegistryBatchCleanupOperation operation2 = Mockito.mock(RegistryBatchCleanupOperation.class);
        Mockito.when(operation2.withOperationName(Mockito.anyString())).thenReturn(operation2);
        RegistryBatchCleanupOperation operation3 = Mockito.mock(RegistryBatchCleanupOperation.class);
        Mockito.when(operation3.withOperationName(Mockito.anyString())).thenReturn(operation3);
        Mockito.when(registryDeletionService.getDeprecatedDocumentsForDeletion(Mockito.any(), Mockito.any(), Mockito.anyInt(), Mockito.anyInt()))
                .thenReturn(Arrays.asList(documentEntry1, documentEntry2, documentEntry3, documentEntry4, documentEntry5));
        Mockito.when(cleanupOperationsFactory.createRegistryBatchCleanupOperation(Mockito.anyList()))
                .thenReturn(operation1).thenReturn(operation2).thenReturn(operation3);

        // Act
        List<StackOperation> stackOperations = deprecatedDocumentPatientIdPrefixCleanupSupplier.run();

        assertEquals(3, stackOperations.size());
        assertEquals(3, stackOperations.size());
        assertTrue(stackOperations.contains(operation1));
        assertTrue(stackOperations.contains(operation2));
        assertTrue(stackOperations.contains(operation3));
        ArgumentCaptor<List> documentEntryCaptor = ArgumentCaptor.forClass(List.class);
        Mockito.verify(cleanupOperationsFactory, Mockito.times(3)).createRegistryBatchCleanupOperation(documentEntryCaptor.capture());
        List documentEntries = (List) documentEntryCaptor.getAllValues().stream().flatMap(entries -> entries.stream())
                .collect(Collectors.toList());
        assertEquals(5, documentEntries.size());
        assertTrue(documentEntries.contains(documentEntry1));
        assertTrue(documentEntries.contains(documentEntry2));
        assertTrue(documentEntries.contains(documentEntry3));
        assertTrue(documentEntries.contains(documentEntry4));
        assertTrue(documentEntries.contains(documentEntry5));
    }
    @Test
    public void run_With0EntriesAndBatchSize2_Returns0BatchOperations() {
        // Arrange
        Mockito.when(registryDeletionService.getDeprecatedDocumentsForDeletion(Mockito.any(), Mockito.any(), Mockito.anyInt(), Mockito.anyInt()))
                .thenReturn(Collections.emptyList());

        // Act
        List<StackOperation> stackOperations = deprecatedDocumentPatientIdPrefixCleanupSupplier.run();

        // Assert
        assertEquals(0, stackOperations.size());
        Mockito.verifyZeroInteractions(cleanupOperationsFactory);
    }

    @Test
    public void run_WhenRetrievingDocumentEntriesFails_ThrowsCleanupException() {
        Mockito.when(registryDeletionService.getDeprecatedDocumentsForDeletion(Mockito.any(), Mockito.any(), Mockito.anyInt(), Mockito.anyInt())).thenThrow(RuntimeException.class);

        CleanupException e = assertThrows(CleanupException.class, () -> deprecatedDocumentPatientIdPrefixCleanupSupplier.run());

        assertTrue(e.getMessage(), e.getMessage().contains("Failed to retrieve deprecated document entries with type codes"));
    }
}
