package dk.nsp.xdscleanup.deprecated.stack.registry.submissionsets;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.xdscleanup.common.model.SubmissionSet;
import dk.nsp.xdscleanup.common.util.CleanupException;
import dk.nsp.xdscleanup.deprecated.factory.CleanupOperationsFactory;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Mockito;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import static dk.nsp.xdscleanup.deprecated.TestUtils.assertThrows;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class DanglingSubmissionSetsCleanupSupplierTest {
    private static final int BATCH_SIZE = 2;
    private static final int LIMIT = 10000;
    private RegistryDeletionService registryDeletionService;
    private CleanupOperationsFactory cleanupOperationsFactory;
    private DanglingSubmissionSetsCleanupSupplier danglingSubmissionSetsCleanupSupplier;

    @Before
    public void setUp() {
        registryDeletionService = Mockito.mock(RegistryDeletionService.class);
        cleanupOperationsFactory = Mockito.mock(CleanupOperationsFactory.class);
        danglingSubmissionSetsCleanupSupplier = new DanglingSubmissionSetsCleanupSupplier(registryDeletionService, cleanupOperationsFactory, BATCH_SIZE, LIMIT);
    }

    @Test
    public void run_With5SubmissionSetsAndBatchSize2_Returns3BatchOperations() {

        SubmissionSet submissionSet1 = new SubmissionSet(1, "1");
        SubmissionSet submissionSet2 = new SubmissionSet(2, "2");
        SubmissionSet submissionSet3 = new SubmissionSet(3, "3");
        SubmissionSet submissionSet4 = new SubmissionSet(4, "4");
        SubmissionSet submissionSet5 = new SubmissionSet(5, "5");
        RegistrySubmissionSetsBatchCleanupOperation operation1 = Mockito.mock(RegistrySubmissionSetsBatchCleanupOperation.class);
        Mockito.when(operation1.withOperationName(Mockito.anyString())).thenReturn(operation1);
        RegistrySubmissionSetsBatchCleanupOperation operation2 = Mockito.mock(RegistrySubmissionSetsBatchCleanupOperation.class);
        Mockito.when(operation2.withOperationName(Mockito.anyString())).thenReturn(operation2);
        RegistrySubmissionSetsBatchCleanupOperation operation3 = Mockito.mock(RegistrySubmissionSetsBatchCleanupOperation.class);
        Mockito.when(operation3.withOperationName(Mockito.anyString())).thenReturn(operation3);
        Mockito.when(registryDeletionService.getDanglingSubmissionSetsForDeletion(LIMIT))
                .thenReturn(Arrays.asList(submissionSet1, submissionSet2, submissionSet3, submissionSet4, submissionSet5));
        Mockito.when(cleanupOperationsFactory.createRegistrySubmissionSetsBatchCleanupOperation(Mockito.anyList()))
                .thenReturn(operation1).thenReturn(operation2).thenReturn(operation3);

        List<StackOperation> stackOperations = danglingSubmissionSetsCleanupSupplier.run();

        assertEquals(3, stackOperations.size());
        assertTrue(stackOperations.contains(operation1));
        assertTrue(stackOperations.contains(operation2));
        assertTrue(stackOperations.contains(operation3));

        ArgumentCaptor<List> submissionSetsCaptor = ArgumentCaptor.forClass(List.class);
        Mockito.verify(cleanupOperationsFactory, Mockito.times(3)).createRegistrySubmissionSetsBatchCleanupOperation(submissionSetsCaptor.capture());
        List submissionSets = (List) submissionSetsCaptor.getAllValues().stream().flatMap(entries -> entries.stream())
                .collect(Collectors.toList());
        assertEquals(5, submissionSets.size());
        assertTrue(submissionSets.contains(submissionSet1));
        assertTrue(submissionSets.contains(submissionSet2));
        assertTrue(submissionSets.contains(submissionSet3));
        assertTrue(submissionSets.contains(submissionSet4));
        assertTrue(submissionSets.contains(submissionSet5));
    }

    @Test
    public void run_With0EntriesAndBatchSize2_Returns0BatchOperations() {
        // Arrange
        Mockito.when(registryDeletionService.getDanglingSubmissionSetsForDeletion(LIMIT))
                .thenReturn(Collections.emptyList());

        // Act
        List<StackOperation> stackOperations = danglingSubmissionSetsCleanupSupplier.run();

        // Assert
        assertEquals(0, stackOperations.size());
        Mockito.verifyZeroInteractions(cleanupOperationsFactory);
    }

    @Test
    public void run_WhenRetrievingDeprecatedDocumentEntriesFails_ThrowsCleanupException() {
        Mockito.when(registryDeletionService.getDanglingSubmissionSetsForDeletion(LIMIT)).thenThrow(RuntimeException.class);

        CleanupException e = assertThrows(CleanupException.class, () -> danglingSubmissionSetsCleanupSupplier.run());

        assertTrue(e.getMessage(), e.getMessage().contains("Failed to retrieve dangling submissionsets"));
    }

}
