package dk.nsp.xdscleanup.deprecated.stack.registry.submissionsets;

import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.xdscleanup.common.model.SubmissionSet;
import dk.nsp.xdscleanup.common.util.CleanupException;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.util.Collections;
import java.util.List;

import static dk.nsp.xdscleanup.deprecated.TestUtils.assertThrows;
import static org.junit.Assert.assertTrue;

public class RegistrySubmissionSetsBatchCleanupOperationTest {
    private RegistryDeletionService registryDeletionService;
    private List<SubmissionSet> submissionSetsForDeletion;
    private RegistrySubmissionSetsBatchCleanupOperation registrySubmissionSetsBatchCleanupOperation;

    @Before
    public void setUp() {
        registryDeletionService = Mockito.mock(RegistryDeletionService.class);
        submissionSetsForDeletion = Collections.singletonList(new SubmissionSet(1, "1"));
        registrySubmissionSetsBatchCleanupOperation = new RegistrySubmissionSetsBatchCleanupOperation(registryDeletionService, submissionSetsForDeletion, new DateTimeProvider());
    }

    @Test
    public void run_WhenDeletionSucceeds_DeletesDocumentEntries() {
        registrySubmissionSetsBatchCleanupOperation.run();

        Mockito.verify(registryDeletionService).deleteSubmissionSets(submissionSetsForDeletion);
    }

    @Test
    public void run_WhenDeletionFails_ThrowsCleanupException() {
        Mockito.doThrow(RuntimeException.class).when(registryDeletionService).deleteSubmissionSets(submissionSetsForDeletion);

        CleanupException e = assertThrows(CleanupException.class, () -> registrySubmissionSetsBatchCleanupOperation.run());
        assertTrue(e.getMessage(), e.getMessage().contains("Failed to delete submissionsets"));
    }
}
