package dk.nsp.xdscleanup.factory;

import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.model.DocumentTypeCleanupSpecification;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import dk.nsp.xdscleanup.openxds.service.RepositoryDeletionService;
import dk.nsp.xdscleanup.sdm.dao.SdmDao;
import dk.nsp.xdscleanup.stack.registry.RegistryBatchCleanupOperation;
import dk.nsp.xdscleanup.stack.registry.status90.Status90RegistryPatientIdBatchCleanupSupplier;
import dk.nsp.xdscleanup.stack.registry.status90.Status90RegistryPatientIdPrefixCleanupSupplier;
import dk.nsp.xdscleanup.stack.registry.timebased.TypeAndTimeBasedRegistryCleanupSupplier;
import dk.nsp.xdscleanup.stack.repository.RepositoryBatchCleanupOperation;

import java.time.Period;
import java.util.List;

public class CleanupOperationsFactory {
    private final RegistryDeletionService registryDeletionService;
    private final RepositoryDeletionService repositoryDeletionService;
    private final SdmDao sdmDao;
    private final DateTimeProvider dateTimeProvider;
    private final int registryBatchSize;
    private final Period status90Period;
    private final int registryTimebasedLimit;

    public CleanupOperationsFactory(RegistryDeletionService registryDeletionService, RepositoryDeletionService repositoryDeletionService, SdmDao sdmDao,
                                    DateTimeProvider dateTimeProvider, int registryBatchSize, Period status90Period, int registryTimebasedLimit) {
        this.registryDeletionService = registryDeletionService;
        this.repositoryDeletionService = repositoryDeletionService;
        this.sdmDao = sdmDao;
        this.dateTimeProvider = dateTimeProvider;
        this.registryBatchSize = registryBatchSize;
        this.status90Period = status90Period;
        this.registryTimebasedLimit = registryTimebasedLimit;
    }

    public RegistryBatchCleanupOperation createRegistryBatchCleanupOperation(List<DocumentEntry> documentEntriesForDeletion) {
        return new RegistryBatchCleanupOperation(registryDeletionService, documentEntriesForDeletion, dateTimeProvider);
    }

    public TypeAndTimeBasedRegistryCleanupSupplier createTypeAndTimeBasedRegistryCleanupSupplier(DocumentTypeCleanupSpecification typeSpecification) {
        return new TypeAndTimeBasedRegistryCleanupSupplier(typeSpecification, registryDeletionService, dateTimeProvider, registryBatchSize, this, registryTimebasedLimit);
    }

    public Status90RegistryPatientIdBatchCleanupSupplier createStatus90RegistryPatientIdBatchCleanupSupplier(List<String> patientIds) {
        return new Status90RegistryPatientIdBatchCleanupSupplier(patientIds, registryDeletionService, sdmDao, dateTimeProvider, this, registryBatchSize, status90Period);
    }

    public Status90RegistryPatientIdPrefixCleanupSupplier createStatus90RegistryPatientIdPrefixCleanupSupplier(String patientIdPrefix) {
        return new Status90RegistryPatientIdPrefixCleanupSupplier(registryDeletionService, this, registryBatchSize, patientIdPrefix);
    }

    public RepositoryBatchCleanupOperation createRepositoryBatchCleanupOperation(List<DocumentEntry> documentEntriesForDeletion) {
        return new RepositoryBatchCleanupOperation(documentEntriesForDeletion, dateTimeProvider, repositoryDeletionService, registryDeletionService);
    }
}
