package dk.nsp.xdscleanup.model;

import dk.nsp.xdscleanup.common.model.xds.TypeCode;

public class DocumentTypeCleanupSpecification {
    private final TypeCode typeCode;

	private final int numberOfMonthsToKeep;
	
	public DocumentTypeCleanupSpecification(TypeCode typeCode, int numberOfMonthsToKeep) {
		this.typeCode = typeCode;
		this.numberOfMonthsToKeep = numberOfMonthsToKeep;
	}

    public TypeCode getTypeCode() {
        return typeCode;
    }

    public int getNumberOfMonthsToKeep() {
        return numberOfMonthsToKeep;
    }

    public static DocumentTypeCleanupSpecification parse(String spec) {
        if(spec == null) {
            indicateInvalidSpec(spec);
        }

        String[] parts = spec.split("#");
        if(parts.length != 3) {
            indicateInvalidSpec(spec);
        }

        String codeName = parts[0];
        String codeScheme = parts[1];
        int numberOfMonthsToKeep = 0;
        try {
            numberOfMonthsToKeep = Integer.parseInt(parts[2]);
            if(numberOfMonthsToKeep <= 0) {
                indicateInvalidSpec(spec);
            }
        }
        catch(NumberFormatException e) {
            indicateInvalidSpec(spec);
        }

        return new DocumentTypeCleanupSpecification(new TypeCode(codeName, codeScheme), numberOfMonthsToKeep);
    }

    private static void indicateInvalidSpec(String spec) {
        throw new IllegalArgumentException(String.format("Not a valid DocumentTypeCleanupSpecification: %s! Proper format is: <code-name>#<code-scheme>#<months-to-keep>", spec));
    }
}
