package dk.nsp.xdscleanup.setup;

import dk.nsp.xdscleanup.nxrg.dao.NxrgDao;
import dk.nsp.xdscleanup.nxrg.dao.impl.NxrgDaoImpl;
import dk.nsp.xdscleanup.openxds.dao.OpenXdsDao;
import dk.nsp.xdscleanup.openxds.dao.impl.OpenXdsDaoImpl;
import dk.nsp.xdscleanup.openxds.health.OpenXdsHealthIndicator;
import dk.nsp.xdscleanup.sdm.dao.SdmDao;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.jdbc.datasource.lookup.JndiDataSourceLookup;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import javax.sql.DataSource;

@Configuration
@EnableTransactionManagement
public class DBSetup {

    @Value("${datasource.nxrg.jndi.name}")
    private String nxrgJndiName;

    @Value("${datasource.openxds.jndi.name}")
    private String openXdsjndiName;

    @Value("${datasource.sdm.jndi.name}")
    private String sdmJndiName;

    @Value("${deletion.max.attempts}")
    private int deletionMaxAttempts;

    @Value("${deletion.in.clause.max.size}")
    private int inClauseMaxSize;

    // Nxrg

    @Bean
    public DataSource nxrgDataSource() {
        return new JndiDataSourceLookup().getDataSource(nxrgJndiName);
    }

    @Bean(name="tx-nxrg")
    public PlatformTransactionManager nxrgTransactionManager(DataSource nxrgDataSource) {
        return new DataSourceTransactionManager(nxrgDataSource);
    }

    @Bean
    public NxrgDao nxrgDao(DataSource nxrgDataSource) {
        return new NxrgDaoImpl(nxrgDataSource, deletionMaxAttempts, inClauseMaxSize);
    }


    // OpenXds

    @Bean
    public DataSource openXdsDataSource() {
        return new JndiDataSourceLookup().getDataSource(openXdsjndiName);
    }

    @Bean(name="tx-openxds")
    public PlatformTransactionManager openXdsTransactionManager(DataSource openXdsDataSource) {
        return new DataSourceTransactionManager(openXdsDataSource);
    }

    @Bean
    public OpenXdsDao openXdsDao(DataSource openXdsDataSource) {
        return new OpenXdsDaoImpl(openXdsDataSource, inClauseMaxSize);
    }

    @Bean
    public OpenXdsHealthIndicator openXdsHealthIndicator(DataSource openXdsDataSource) {
        return new OpenXdsHealthIndicator(new JdbcTemplate(openXdsDataSource));
    }

    // SDM

    @Bean(name = "sdmDataSource")
    public DataSource sdmDataSource() {
        return new JndiDataSourceLookup().getDataSource(sdmJndiName);
    }

    @Bean
    public SdmDao sdmDao(DataSource sdmDataSource) {
        return new SdmDao(sdmDataSource, inClauseMaxSize);
    }
}
