package dk.nsp.xdscleanup.stack.registry.status90;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.common.util.CleanupUtils;
import dk.nsp.xdscleanup.factory.CleanupOperationsFactory;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import dk.nsp.xdscleanup.common.util.CleanupException;
import dk.nsp.xdscleanup.sdm.dao.SdmDao;

import java.time.Instant;
import java.time.Period;
import java.util.List;
import java.util.stream.Collectors;

public class Status90RegistryPatientIdBatchCleanupSupplier implements StackOperation {
    private final List<String> cprs;
    private final RegistryDeletionService registryDeletionService;
    private final SdmDao sdmDao;
    private final DateTimeProvider dateTimeProvider;
    private final CleanupOperationsFactory cleanupOperationsFactory;
    private final int batchSize;
    private final Period status90Period;

    public Status90RegistryPatientIdBatchCleanupSupplier(List<String> cprs, RegistryDeletionService registryDeletionService, SdmDao sdmDao,
                                                         DateTimeProvider dateTimeProvider, CleanupOperationsFactory cleanupOperationsFactory, int batchSize,
                                                         Period status90Period) {
        this.cprs = cprs;
        this.registryDeletionService = registryDeletionService;
        this.sdmDao = sdmDao;
        this.dateTimeProvider = dateTimeProvider;
        this.cleanupOperationsFactory = cleanupOperationsFactory;
        this.batchSize = batchSize;
        this.status90Period = status90Period;
    }

    @Override
    public List<StackOperation> run() {
        Instant latestStatus90Time = dateTimeProvider.now().minus(status90Period).toInstant();
        List<String> status90Citizens = getStatus90Citizens(latestStatus90Time);
        List<DocumentEntry> documentEntriesForDeletion = getDocumentEntriesForCitizens(status90Citizens);

        return CleanupUtils.createShuffledBatches(documentEntriesForDeletion, batchSize).stream()
                .map(batch -> cleanupOperationsFactory.createRegistryBatchCleanupOperation(batch).withOperationName("Registry Status 90 Cleanup"))
                .collect(Collectors.toList());
    }

    private List<String> getStatus90Citizens(Instant latestStatus90Time) {
        try {
            return sdmDao.getStatus90Citizens(cprs, latestStatus90Time);
        } catch (Exception e) {
            throw new CleanupException("Failed to retrieve status 90 citizens with latestStatusTime " + latestStatus90Time, e);
        }
    }

    private List<DocumentEntry> getDocumentEntriesForCitizens(List<String> status90Citizens) {
        try {
            return registryDeletionService.getDocumentEntriesForCitizens(status90Citizens);
        } catch (Exception e) {
            throw new CleanupException("Failed to retrieve document entries for citizens", e);
        }
    }
}
