package dk.nsp.xdscleanup.stack.registry.status90;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.xdscleanup.common.util.CleanupUtils;
import dk.nsp.xdscleanup.factory.CleanupOperationsFactory;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import dk.nsp.xdscleanup.common.util.CleanupException;

import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

public class Status90RegistryPatientIdPrefixCleanupSupplier implements StackOperation {
    private final RegistryDeletionService registryDeletionService;
    private final CleanupOperationsFactory cleanupOperationsFactory;
    private final int batchSize;
    private final String patientIdPrefix;

    public Status90RegistryPatientIdPrefixCleanupSupplier(RegistryDeletionService registryDeletionService, CleanupOperationsFactory cleanupOperationsFactory,
                                                          int batchSize, String patientIdPrefix) {
        this.registryDeletionService = registryDeletionService;
        this.cleanupOperationsFactory = cleanupOperationsFactory;
        this.batchSize = batchSize;
        this.patientIdPrefix = patientIdPrefix;
    }

    @Override
    public List<StackOperation> run() {
        List<String> patientIds = getDocumentEntryPatientIdsWithPrefix();
        Collection<List<String>> patientIdBatches = CleanupUtils.createShuffledBatches(patientIds, batchSize);
        return patientIdBatches.stream()
                .map(cleanupOperationsFactory::createStatus90RegistryPatientIdBatchCleanupSupplier)
                .collect(Collectors.toList());
    }

    private List<String> getDocumentEntryPatientIdsWithPrefix() {
        try {
            return registryDeletionService.getDocumentEntryPatientIdsWithPrefix(patientIdPrefix);
        } catch (Exception e) {
            throw new CleanupException("Failed to retrieve document entry patient ids with prefix " + patientIdPrefix, e);
        }
    }
}
