package dk.nsp.xdscleanup.stack.repository;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.common.util.CleanupUtils;
import dk.nsp.xdscleanup.factory.CleanupOperationsFactory;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import dk.nsp.xdscleanup.common.util.CleanupException;

import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

public class RepositoryCleanupSupplier implements StackOperation {
    private final RegistryDeletionService registryDeletionService;
    private final CleanupOperationsFactory cleanupOperationsFactory;
    private final int batchSize;
    private final int repositoryLimit;

    public RepositoryCleanupSupplier(RegistryDeletionService registryDeletionService, CleanupOperationsFactory cleanupOperationsFactory, int batchSize, int repositoryLimit) {
        this.registryDeletionService = registryDeletionService;
        this.cleanupOperationsFactory = cleanupOperationsFactory;
        this.batchSize = batchSize;
        this.repositoryLimit = repositoryLimit;
    }

    @Override
    public List<StackOperation> run() {
        List<DocumentEntry> deletedDocumentEntries = findDeletedDocumentEntries(repositoryLimit);
        Collection<List<DocumentEntry>> entryBatches = CleanupUtils.createShuffledBatches(deletedDocumentEntries, batchSize);

        return entryBatches.stream()
                .map(cleanupOperationsFactory::createRepositoryBatchCleanupOperation)
                .collect(Collectors.toList());
    }

    private List<DocumentEntry> findDeletedDocumentEntries(int limit) {
        try {
            return registryDeletionService.findDeletedDocumentEntries(limit);
        } catch (Exception e) {
            throw new CleanupException("Failed to retrieve deleted document entries", e);
        }
    }
}
