package dk.nsp.xdscleanup.stack.registry;

import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import dk.nsp.xdscleanup.common.util.CleanupException;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.util.Collections;
import java.util.List;

import static dk.nsp.xdscleanup.TestUtils.assertThrows;
import static org.junit.Assert.assertTrue;

public class RegistryBatchCleanupOperationTest {

    private RegistryDeletionService registryDeletionService;
    private List<DocumentEntry> documentEntriesForDeletion;
    private RegistryBatchCleanupOperation registryBatchCleanupOperation;

    @Before
    public void setUp() throws Exception {
        registryDeletionService = Mockito.mock(RegistryDeletionService.class);
        documentEntriesForDeletion = Collections.singletonList(new DocumentEntry(1, "1", "1"));
        registryBatchCleanupOperation = new RegistryBatchCleanupOperation(registryDeletionService, documentEntriesForDeletion, new DateTimeProvider());
    }

    @Test
    public void run_WhenDeletionSucceeds_DeletesDocumentEntries() {
        registryBatchCleanupOperation.run();

        Mockito.verify(registryDeletionService).deleteDocumentEntries(documentEntriesForDeletion);
    }

    @Test
    public void run_WhenDeletionFails_ThrowsCleanupException() {
        Mockito.doThrow(RuntimeException.class).when(registryDeletionService).deleteDocumentEntries(documentEntriesForDeletion);

        CleanupException e = assertThrows(CleanupException.class, () -> registryBatchCleanupOperation.run());
        assertTrue(e.getMessage(), e.getMessage().contains("Failed to delete document entries"));
    }
}