package dk.nsp.xdscleanup.stack.registry.timebased;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.xdscleanup.common.model.xds.TypeCode;
import dk.nsp.xdscleanup.factory.CleanupOperationsFactory;
import dk.nsp.xdscleanup.model.DocumentTypeCleanupSpecification;
import org.junit.Test;
import org.mockito.Mockito;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class TypeSpecificationBasedRegistryCleanupSupplierTest {

    @Test
    public void run_With2TypeSpecifications_ReturnsExpectedSuppliersForTypeSpec() {
        // Arrange
        DocumentTypeCleanupSpecification typeSpec1 = new DocumentTypeCleanupSpecification(new TypeCode("Code1", "1"), 1);
        DocumentTypeCleanupSpecification typeSpec2 = new DocumentTypeCleanupSpecification(new TypeCode("Code2", "2"), 2);
        TypeAndTimeBasedRegistryCleanupSupplier supplier1 = Mockito.mock(TypeAndTimeBasedRegistryCleanupSupplier.class);
        TypeAndTimeBasedRegistryCleanupSupplier supplier2 = Mockito.mock(TypeAndTimeBasedRegistryCleanupSupplier.class);
        CleanupOperationsFactory cleanupOperationsFactory = Mockito.mock(CleanupOperationsFactory.class);
        Mockito.when(cleanupOperationsFactory.createTypeAndTimeBasedRegistryCleanupSupplier(typeSpec1))
                .thenReturn(supplier1);
        Mockito.when(cleanupOperationsFactory.createTypeAndTimeBasedRegistryCleanupSupplier(typeSpec2))
                .thenReturn(supplier2);
        TypeSpecificationBasedRegistryCleanupSupplier typeSpecBasedSupplier = new TypeSpecificationBasedRegistryCleanupSupplier(
                Arrays.asList(typeSpec1, typeSpec2), cleanupOperationsFactory);

        // Act
        List<StackOperation> stackOperations = typeSpecBasedSupplier.run();

        // Assert
        assertEquals(2, stackOperations.size());
        assertTrue(stackOperations.contains(supplier1));
        assertTrue(stackOperations.contains(supplier2));
    }

    @Test
    public void run_With0TypeSpecifications_ReturnsEmptyList() {
        // Arrange
        CleanupOperationsFactory cleanupOperationsFactory = Mockito.mock(CleanupOperationsFactory.class);
        TypeSpecificationBasedRegistryCleanupSupplier typeSpecBasedSupplier = new TypeSpecificationBasedRegistryCleanupSupplier(
                Collections.emptyList(), cleanupOperationsFactory);

        // Act
        List<StackOperation> stackOperations = typeSpecBasedSupplier.run();

        // Assert
        assertEquals(0, stackOperations.size());
    }
}