package dk.nsp.xdscleanup.stack.repository;

import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import dk.nsp.xdscleanup.openxds.service.RepositoryDeletionService;
import dk.nsp.xdscleanup.common.util.CleanupException;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.List;

import static dk.nsp.xdscleanup.TestUtils.assertThrows;
import static org.junit.Assert.assertTrue;

public class RepositoryBatchCleanupOperationTest {

    private RepositoryDeletionService repositoryDeletionService;
    private RegistryDeletionService registryDeletionService;
    private List<DocumentEntry> documentEntries;
    private RepositoryBatchCleanupOperation repositoryBatchCleanupOperation;

    @Before
    public void setUp() throws Exception {
        DateTimeProvider dateTimeProvider = Mockito.mock(DateTimeProvider.class);
        Mockito.when(dateTimeProvider.now()).thenReturn(ZonedDateTime.now());
        repositoryDeletionService = Mockito.mock(RepositoryDeletionService.class);
        registryDeletionService = Mockito.mock(RegistryDeletionService.class);
        documentEntries = Arrays.asList(new DocumentEntry(1, "1", "1"), new DocumentEntry(2, "2", "2"));
        repositoryBatchCleanupOperation = new RepositoryBatchCleanupOperation(documentEntries, dateTimeProvider, repositoryDeletionService, registryDeletionService);
    }

    @Test
    public void run_DeletesDocuments() {
        repositoryBatchCleanupOperation.run();

        Mockito.verify(repositoryDeletionService).deleteDocuments(documentEntries);
    }

    @Test
    public void run_WhenDeletionSucceeds_MarksDeletionComplete() {
        repositoryBatchCleanupOperation.run();

        Mockito.verify(registryDeletionService).markDeletionComplete(documentEntries);
    }

    @Test
    public void run_WhenDeletionFails_MarksDeletionFailedAndThrows() {
        Mockito.when(repositoryDeletionService.deleteDocuments(documentEntries)).thenThrow(RuntimeException.class);

        CleanupException e = assertThrows(CleanupException.class, () -> repositoryBatchCleanupOperation.run());
        assertTrue(e.getMessage().contains("Failed to delete documents"));

        Mockito.verify(registryDeletionService).markDeletionAttemptFailed(documentEntries);
    }
}