package dk.nsp.xdscleanup.nxrg.dao;

import dk.nsp.xdscleanup.common.model.Association;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.common.model.SubmissionSet;
import dk.nsp.xdscleanup.common.model.xds.TypeCode;

import java.time.Instant;
import java.util.List;

public interface NxrgDao {
    List<DocumentEntry> getDocumentEntriesForCitizens(List<String> citizenCprs);

    List<DocumentEntry> getDocumentEntriesForDeletion(TypeCode typeCode, Instant latestStartTime, int limit);

    List<DocumentEntry> getDeprecatedDocumentsForDeletion(String patientIdPrefix, TypeCode typeCode, int daysBeforeDelete, int limit);

    List<SubmissionSet> getDanglingSubmissionSetsForDeletion(int limit);

    List<String> getDocumentEntryPatientIdsWithPrefix(String prefix);

    List<Association> getAssociationsForDocumentEntries(List<String> documentEntryUuids);

    /**
     * Takes a list of SubmissionSet entryUuid's, determines which of those SubmissionSets will be empty after deleting
     * the DocumentEntries denoted by documentEntryUuidsForDeletion.
     * @param submissionSetEntryUuids
     * @param documentEntryUuidsForDeletion
     * @return
     */
    List<SubmissionSet> determineSubmissionSetsLeftEmpty(List<String> submissionSetEntryUuids, List<String> documentEntryUuidsForDeletion);

    /**
     * Delete the DocumentEntries identified by the provided list of id's.
     * Deletes from the documententries-table and related tables: documententrycontent, documententry_author,
     * documententry_eventcode, documententry_confidentialitycode and documententry_referenceid.
     * @param documentEntryIds The entries to delete.
     */
    void deleteDocumentEntries(List<Long> documentEntryIds);

    /**
     * Delete the Associations identified by the provided list of id's.
     * Deletes from the associations-table and the related associationcontent-table.
     * @param associationIds The associations to delete.
     */
    void deleteAssociations(List<Long> associationIds);

    /**
     * Delete the SubmissionSets identified by the provided list of id's.
     * Deletes from the submissionsets-table and the related submissionsetcontent-table.
     * @param submissionSetIds The submission sets to delete.
     */
    void deleteSubmissionSets(List<Long> submissionSetIds);

    List<DocumentEntry> getDocumentEntriesForRepositoryDeletion(int limit);

    /**
     * Adds entries to the deleted_documententries table with status DELETED_FROM_REGISTRY
     */
    void markSuccessfulRegistryDeletion(List<DocumentEntry> deletedDocumentEntries);

    /**
     * Removes entries from the deleted_documententries table
     */
    void markSuccessfulRepositoryDeletion(List<String> documentEntryUuids);

    /**
     * Updates entries in the deleted_documententries table with status DELETION_FROM_REPOSITORY_FAILED
     */
    void markFailedRepositoryDeletion(List<String> documentEntryUuids);
}
