package dk.nsp.xdscleanup.sdm.dao;

import dk.nsp.backgroundjob.health.HealthIndication;
import dk.nsp.backgroundjob.health.HealthIndicator;
import dk.nsp.xdscleanup.common.util.DateTimeUtils;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;

import javax.sql.DataSource;
import java.time.Instant;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class SdmDao implements HealthIndicator {
    private final DataSource dataSource;
    private final int inClauseMaxSize;

    public SdmDao(DataSource dataSource, int inClauseMaxSize) {
        this.dataSource = dataSource;
        this.inClauseMaxSize = inClauseMaxSize;
    }

    public List<String> getStatus90Citizens(List<String> citizenCprs, Instant latestStatusTime) {
        if (citizenCprs.isEmpty()) {
            return Collections.emptyList();
        }

        List<String> status90Citizens = new ArrayList<>();
        // The list may be long, so we split it into chunks.
        for(int i = 0; i < citizenCprs.size(); i += inClauseMaxSize) {
            status90Citizens.addAll(getStatus90CitizensBatch(citizenCprs.subList(i, Math.min(i + inClauseMaxSize, citizenCprs.size())), latestStatusTime));
        }
        return status90Citizens;
    }

    private List<String> getStatus90CitizensBatch(List<String> citizenCprs, Instant latestStatusTime) {
        String query = "select CPR from persons_status90_v" +
                "       where CPR in (:cprs) and StatusDato < :latestStatusTime";

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("cprs", citizenCprs);
        parameters.addValue("latestStatusTime", DateTimeUtils.format(latestStatusTime));

        NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
        return template.queryForList(query, parameters, String.class);
    }

    @Override
    public HealthIndication getHealthInfo() {
        String sdmDb = "sdm.db";
        String query = "select 1 from persons_status90_v";

        MapSqlParameterSource parameters = new MapSqlParameterSource();

        try {
            NamedParameterJdbcTemplate template = new NamedParameterJdbcTemplate(dataSource);
            template.query(query, parameters, (rs, rowNum) -> rs.getString(1));
            return new HealthIndication(200, "\""+sdmDb+"\": \"OK\"");
        }
        catch (DataAccessException e) {
            return new HealthIndication(500, "\""+sdmDb+"\": \"ERROR: Database connection dead.\"");
        }
    }
}
