package dk.nsp.xdscleanup.nxrg.dao;

import dk.nsp.xdscleanup.common.model.DeletionStatus;
import dk.nsp.xdscleanup.common.model.xds.AssociationType;
import dk.nsp.xdscleanup.common.model.xds.ReferenceType;
import dk.nsp.xdscleanup.common.model.xds.TypeCode;
import dk.nsp.xdscleanup.common.util.DateTimeUtils;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;

import javax.sql.DataSource;
import java.time.Duration;
import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.UUID;

public class NxrgTestDataCreator {
    private NamedParameterJdbcTemplate template;

    public NxrgTestDataCreator(DataSource dataSource) {
        this.template = new NamedParameterJdbcTemplate(dataSource);
    }

    public void initializeTestDatabase() {
        // Delete everything
        template.update("delete from associationcontent", new MapSqlParameterSource());
        template.update("delete from documententrycontent", new MapSqlParameterSource());
        template.update("delete from documententries", new MapSqlParameterSource());
        template.update("delete from foldercontent", new MapSqlParameterSource());
        template.update("delete from submissionsetcontent", new MapSqlParameterSource());
        template.update("delete from deleted_documententries", new MapSqlParameterSource());

        // Insert test data
        insertDocumentEntry(TestConstants.DOCUMENT_ENTRY_UUID_0, TestConstants.TYPE_CODE_0, TestConstants.SERVICE_START_TIME_0, TestConstants.CREATION_TIME_0, TestConstants.CPR_1, TestConstants.STATUS_APPROVED);
        insertDocumentEntry(TestConstants.DOCUMENT_ENTRY_UUID_1, TestConstants.TYPE_CODE_1, TestConstants.SERVICE_START_TIME_1, TestConstants.CREATION_TIME_1, TestConstants.CPR_1, TestConstants.STATUS_APPROVED);
        insertDocumentEntry(TestConstants.DOCUMENT_ENTRY_UUID_2, TestConstants.TYPE_CODE_0, TestConstants.SERVICE_START_TIME_0, TestConstants.CREATION_TIME_0, TestConstants.CPR_1, TestConstants.STATUS_APPROVED);
        insertDocumentEntry(TestConstants.DOCUMENT_ENTRY_UUID_3, TestConstants.TYPE_CODE_0, null, TestConstants.CREATION_TIME_0, TestConstants.CPR_1, TestConstants.STATUS_APPROVED);

        // Don't add an association for this one: It exists for testing the delete-method.
        insertDocumentEntry(TestConstants.DOCUMENT_ENTRY_UUID_ORPHAN, TestConstants.TYPE_CODE_ORPHAN, null, TestConstants.CREATION_TIME_ORPHAN, TestConstants.CPR_1, TestConstants.STATUS_APPROVED);

        insertSubmissionSet(TestConstants.SUBMISSION_SET_ENTRY_UUID_0);
        insertSubmissionSet(TestConstants.SUBMISSION_SET_ENTRY_UUID_1);
        insertSubmissionSet(TestConstants.SUBMISSION_SET_ENTRY_UUID_2);
        insertSubmissionSet(TestConstants.SUBMISSION_SET_ENTRY_UUID_3);
        insertSubmissionSet(TestConstants.SUBMISSION_SET_ENTRY_UUID_4); //anvendes til at teste slet af submissionset og submissioncontent. Må ikke have foreign key relationer

        // Don't add an association for this one: It exists for testing the delete-method.
        insertSubmissionSet(TestConstants.SUBMISSION_SET_ENTRY_UUID_ORPHAN);

        insertAssociation(TestConstants.ASSOCIATION_ENTRY_UUID_0, AssociationType.HAS_MEMBER, TestConstants.SUBMISSION_SET_ENTRY_UUID_0, ReferenceType.SUBMISSION_SET, TestConstants.DOCUMENT_ENTRY_UUID_0, ReferenceType.DOCUMENT_ENTRY, TestConstants.CREATION_TIME_0);
        insertAssociation(TestConstants.ASSOCIATION_ENTRY_UUID_1, AssociationType.HAS_MEMBER, TestConstants.SUBMISSION_SET_ENTRY_UUID_1, ReferenceType.SUBMISSION_SET, TestConstants.DOCUMENT_ENTRY_UUID_1, ReferenceType.DOCUMENT_ENTRY, TestConstants.CREATION_TIME_0);
        insertAssociation(TestConstants.ASSOCIATION_ENTRY_UUID_2, AssociationType.HAS_MEMBER, TestConstants.SUBMISSION_SET_ENTRY_UUID_2, ReferenceType.SUBMISSION_SET, TestConstants.DOCUMENT_ENTRY_UUID_2, ReferenceType.DOCUMENT_ENTRY, TestConstants.CREATION_TIME_0);
        // NB: SubmissionSet 2 contains DocumentEntries 2 and 3!
        insertAssociation(TestConstants.ASSOCIATION_ENTRY_UUID_3, AssociationType.HAS_MEMBER, TestConstants.SUBMISSION_SET_ENTRY_UUID_2, ReferenceType.SUBMISSION_SET, TestConstants.DOCUMENT_ENTRY_UUID_3, ReferenceType.DOCUMENT_ENTRY, TestConstants.CREATION_TIME_0);

        insertAssociation(TestConstants.ASSOCIATION_ENTRY_UUID_4, AssociationType.REPLACE, TestConstants.DOCUMENT_ENTRY_UUID_0, ReferenceType.DOCUMENT_ENTRY, TestConstants.DOCUMENT_ENTRY_UUID_1, ReferenceType.DOCUMENT_ENTRY, TestConstants.CREATION_TIME_0);

        insertDeletedDocumentEntry(TestConstants.DELETED_DOCUMENT_ENTRY_UUID_1, DeletionStatus.DELETION_FROM_REPOSITORY_FAILED, 2);
        insertDeletedDocumentEntry(TestConstants.DELETED_DOCUMENT_ENTRY_UUID_2, DeletionStatus.DELETION_FROM_REPOSITORY_FAILED, 1);
        insertDeletedDocumentEntry(TestConstants.DELETED_DOCUMENT_ENTRY_UUID_3, DeletionStatus.DELETED_FROM_REGISTRY, 0);

        insertDocumentEntry(TestConstants.DOCUMENT_ENTRY_UUID_4, TestConstants.TYPE_CODE_2, TestConstants.SERVICE_START_TIME_2, TestConstants.CREATION_TIME_0, TestConstants.CPR_2, TestConstants.STATUS_APPROVED);
        insertDocumentEntry(TestConstants.DOCUMENT_ENTRY_UUID_5, TestConstants.TYPE_CODE_2, TestConstants.SERVICE_START_TIME_2, TestConstants.CREATION_TIME_0, TestConstants.CPR_3, TestConstants.STATUS_APPROVED);

        insertDocumentEntry(TestConstants.DOCUMENT_ENTRY_UUID_6, TestConstants.TYPE_CODE_2, TestConstants.SERVICE_START_TIME_2, TestConstants.CREATION_TIME_0, TestConstants.CPR_4, TestConstants.STATUS_APPROVED);
        insertDocumentEntry(TestConstants.DOCUMENT_ENTRY_UUID_7, TestConstants.TYPE_CODE_2, TestConstants.SERVICE_START_TIME_2, TestConstants.CREATION_TIME_0, TestConstants.CPR_5, TestConstants.STATUS_APPROVED);
        insertDocumentEntry(TestConstants.DOCUMENT_ENTRY_UUID_8, TestConstants.TYPE_CODE_2, TestConstants.SERVICE_START_TIME_2, TestConstants.CREATION_TIME_0, TestConstants.CPR_6, TestConstants.STATUS_APPROVED);

        // Insert deprecated documents
        insertDocumentEntry(TestConstants.DOCUMENT_ENTRY_UUID_9, TestConstants.TYPE_CODE_2, TestConstants.SERVICE_START_TIME_2, Instant.now().truncatedTo(ChronoUnit.DAYS).minus(Duration.ofDays(61)), TestConstants.CPR_6, TestConstants.STATUS_DEPRECATED);
        insertDocumentEntry(TestConstants.DOCUMENT_ENTRY_UUID_10, TestConstants.TYPE_CODE_2, TestConstants.SERVICE_START_TIME_2, Instant.now().truncatedTo(ChronoUnit.DAYS).minus(Duration.ofDays(60)), TestConstants.CPR_6, TestConstants.STATUS_DEPRECATED);
        insertDocumentEntry(TestConstants.DOCUMENT_ENTRY_UUID_11, TestConstants.TYPE_CODE_2, TestConstants.SERVICE_START_TIME_2, Instant.now().truncatedTo(ChronoUnit.DAYS).minus(Duration.ofDays(59)), TestConstants.CPR_6, TestConstants.STATUS_DEPRECATED);

        insertAssociation(TestConstants.ASSOCIATION_ENTRY_UUID_7, AssociationType.REPLACE, TestConstants.DOCUMENT_ENTRY_UUID_10, ReferenceType.DOCUMENT_ENTRY, TestConstants.DOCUMENT_ENTRY_UUID_9, ReferenceType.DOCUMENT_ENTRY, Instant.now().truncatedTo(ChronoUnit.DAYS).minus(Duration.ofDays(61)));
        insertAssociation(TestConstants.ASSOCIATION_ENTRY_UUID_8, AssociationType.REPLACE, TestConstants.DOCUMENT_ENTRY_UUID_11, ReferenceType.DOCUMENT_ENTRY, TestConstants.DOCUMENT_ENTRY_UUID_10, ReferenceType.DOCUMENT_ENTRY, Instant.now().truncatedTo(ChronoUnit.DAYS).minus(Duration.ofDays(60)));

        insertAssociation(TestConstants.ASSOCIATION_ENTRY_UUID_5, AssociationType.UPDATE_AVAILABILITY_STATUS, TestConstants.SUBMISSION_SET_ENTRY_UUID_3, ReferenceType.SUBMISSION_SET, TestConstants.DOCUMENT_ENTRY_UUID_9, ReferenceType.DOCUMENT_ENTRY, Instant.now().truncatedTo(ChronoUnit.DAYS).minus(Duration.ofDays(61)));
        insertAssociation(TestConstants.ASSOCIATION_ENTRY_UUID_6, AssociationType.HAS_MEMBER, TestConstants.SUBMISSION_SET_ENTRY_UUID_3, ReferenceType.SUBMISSION_SET, TestConstants.DOCUMENT_ENTRY_UUID_9, ReferenceType.DOCUMENT_ENTRY, Instant.now().truncatedTo(ChronoUnit.DAYS).minus(Duration.ofDays(61)));
    }

    public void insertDeletedDocumentEntry(String entryUuid, DeletionStatus deletionStatus, int deletionAttempts) {
        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("entryUuid", entryUuid);
        parameters.addValue("uniqueId", UUID.randomUUID().toString());
        parameters.addValue("deletionStatus", deletionStatus.toString());
        parameters.addValue("deletionAttempts", deletionAttempts);

        template.update("insert into deleted_documententries(entryuuid, uniqueid, deletion_status, deletion_attempts) values (:entryUuid, :uniqueId, :deletionStatus, :deletionAttempts)", parameters);
    }

    private void insertAssociation(String entryUuid, AssociationType associationType, String sourceUuid, ReferenceType sourceType, String targetUuid, ReferenceType targetType, Instant creationTime) {
        KeyHolder keyHolder = new GeneratedKeyHolder();
        template.update("insert into associationcontent(xml) values('bla')", new MapSqlParameterSource(), keyHolder, new String[] { "id" });

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("entryUuid", entryUuid);
        parameters.addValue("associationType", associationType.toString());

        parameters.addValue("sourceUuidAssociation", sourceType == ReferenceType.ASSOCIATION ? sourceUuid : null);
        parameters.addValue("sourceUuidDocumentEntry", sourceType == ReferenceType.DOCUMENT_ENTRY ? sourceUuid : null);
        parameters.addValue("sourceUuidFolder", sourceType == ReferenceType.FOLDER ? sourceUuid : null);
        parameters.addValue("sourceUuidSubmissionSet", sourceType == ReferenceType.SUBMISSION_SET ? sourceUuid : null);

        parameters.addValue("targetUuidAssociation", targetType == ReferenceType.ASSOCIATION ? targetUuid : null);
        parameters.addValue("targetUuidDocumentEntry", targetType == ReferenceType.DOCUMENT_ENTRY ? targetUuid : null);
        parameters.addValue("targetUuidFolder", targetType == ReferenceType.FOLDER ? targetUuid : null);
        parameters.addValue("targetUuidSubmissionSet", targetType == ReferenceType.SUBMISSION_SET ? targetUuid : null);

        parameters.addValue("associationContentId", keyHolder.getKey().longValue());

        String creationTimeStr = DateTimeUtils.format(creationTime);
        parameters.addValue("creationTime", creationTimeStr);

        template.update("insert into associations (entryuuid, association_type, source_association, sourceuuid_documententry, sourceuuid_folder, sourceuuid_submissionset, target_association, targetuuid_documententry, targetuuid_folder, targetuuid_submissionset, associationcontentid, creation_time) values (:entryUuid, :associationType, null, :sourceUuidDocumentEntry, null, :sourceUuidSubmissionSet, null, :targetUuidDocumentEntry, null, :targetUuidSubmissionSet, :associationContentId, :creationTime)", parameters);
    }

    private void insertDocumentEntry(String entryUuid, TypeCode typeCode, Instant serviceStartTime, Instant creationTime, String patientid, String availabilitystatus) {
        KeyHolder keyHolder = new GeneratedKeyHolder();
        template.update("insert into documententrycontent(xml) values('bla')", new MapSqlParameterSource(), keyHolder, new String[] { "id" });

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("entryUuid", entryUuid);
        parameters.addValue("typeCodeCodeName", typeCode.getCodeName());
        parameters.addValue("typeCodeSchemeName", typeCode.getCodeScheme());
        String serviceStartTimeStr = DateTimeUtils.format(serviceStartTime);
        parameters.addValue("serviceStartTime", serviceStartTimeStr);
        String creationTimeStr = DateTimeUtils.format(creationTime);
		parameters.addValue("creationTime", creationTimeStr);
        parameters.addValue("documentEntryContentId", keyHolder.getKey().longValue());
        parameters.addValue("patientid", patientid);
        parameters.addValue("availabilitystatus", availabilitystatus);
        String deleteTriggerTime = serviceStartTimeStr;
        if (deleteTriggerTime == null) {
        	deleteTriggerTime = creationTimeStr;
        }
		parameters.addValue("deletetriggerTime", deleteTriggerTime);

        template.update("insert into documententries "+
                        "(entryuuid, logicaluuid, version, patientid, patientid_assigningauthorityid, patientid_assigningauthoritytype, uniqueid, documententrycontentid, creationtime, formatcode_codename, formatcode_schemename, availabilitystatus, documententrytype, classcode_codename, classcode_schemename, typecode_codename, typecode_schemename, servicestarttime, servicestoptime, healthcarefacilitytypecode_codename, healthcarefacilitytypecode_schemename, practicesettingcode_codename, practicesettingcode_schemename, creation_time, deletetrigger_time) "+
                        "values "+
                        "(:entryUuid, 'c80bb5e1-1b5a-4240-9a41-6336870e0012', 1, :patientid, '1.2.208.176.1.2', 'ISO2', '111110012', :documentEntryContentId, '2021-06-09', 'foo', 'bar', :availabilitystatus, 'urn:uuid:34268e47-fdf5-41a6-ba33-82133c465248', 'CC', 'bar', :typeCodeCodeName, :typeCodeSchemeName, :serviceStartTime, '2027-06-05', 'HFTC', 'bar', 'PSC', 'bar', :creationTime, :deletetriggerTime)", parameters);
    }

    private void insertSubmissionSet(String entryUuid) {
        KeyHolder keyHolder = new GeneratedKeyHolder();
        template.update("insert into submissionsetcontent(xml) values('bla')", new MapSqlParameterSource(), keyHolder, new String[] { "id" });

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("entryUuid", entryUuid);
        parameters.addValue("uniqueId", UUID.randomUUID().toString());
        parameters.addValue("submissionSetContentId", keyHolder.getKey().longValue());

        template.update("insert into submissionsets(entryuuid, patientid, patientid_assigningauthorityid, patientid_assigningauthoritytype, uniqueid, submissionsetcontentid) values(:entryUuid, '1212121212', '1.2.208.176.1.2', 'ISO', :uniqueId, :submissionSetContentId)", parameters);
    }

}
