package dk.nsp.xdscleanup.sdm.dao;

import dk.nsp.xdscleanup.nxrg.setup.EmbeddedDatabaseSetup;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import javax.sql.DataSource;
import java.time.Instant;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static dk.nsp.xdscleanup.nxrg.dao.TestConstants.*;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
@RunWith(SpringRunner.class)
@ContextConfiguration(classes = {EmbeddedDatabaseSetup.class})
public class SdmDaoTest {
    private static final int IN_CLAUSE_MAX_SIZE = 1;

    private SdmDao subject;

    @Autowired
    private DataSource dataSource;

    @Before
    public void setUp() throws Exception {
        new SdmTestDataCreator(dataSource).initializeTestDatabase();
        subject = new SdmDao(dataSource, IN_CLAUSE_MAX_SIZE);
    }

    @Test
    public void getStatus90Citizens_WithNonExistingCitizen_ReturnsEmptyList() {
        String nonExistingCitizen = "0000000000";

        List<String> status90Citizens = subject.getStatus90Citizens(Collections.singletonList(nonExistingCitizen), Instant.now());

        assertTrue(status90Citizens.isEmpty());
    }

    @Test
    public void getStatus90Citizens_WithEmptyCitizenList_ReturnsEmptyList() {
        List<String> status90Citizens = subject.getStatus90Citizens(Collections.emptyList(), Instant.now());

        assertTrue(status90Citizens.isEmpty());
    }

    @Test
    public void getStatus90Citizens_WithOneOutOfThreeStatus90Citizens_ReturnsStatus90Citizen() {
        String nonExistingCitizen = "0000000000";
        String status90Citizen = CPR_2;
        String futureStatus90Citizen = CPR_6;

        List<String> status90Citizens = subject.getStatus90Citizens(Arrays.asList(nonExistingCitizen, status90Citizen, futureStatus90Citizen), Instant.now());

        assertEquals(Collections.singletonList(status90Citizen), status90Citizens);
    }

    @Test
    public void getStatus90Citizens_WithThreeOutOfThreeStatus90Citizens_ReturnsStatus90Citizen() {
        List<String> status90Citizens = Arrays.asList(CPR_2, CPR_3, CPR_4);

        List<String> result = subject.getStatus90Citizens(status90Citizens, Instant.now());

        assertEquals(status90Citizens.size(), result.size());
        assertTrue(result.containsAll(status90Citizens));
    }
}