package dk.nsp.xdscleanup.openxds.service;

import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.openxds.dao.OpenXdsDao;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

public class RepositoryDeletionService {
    private OpenXdsDao openXdsDao;

    public RepositoryDeletionService(OpenXdsDao openXdsDao) {
        this.openXdsDao = openXdsDao;
    }

    @Transactional(transactionManager = "tx-openxds")
    public List<DocumentEntry> deleteDocuments(List<DocumentEntry> deletedDocumentEntries) {
        if(deletedDocumentEntries == null || deletedDocumentEntries.isEmpty()) {
            return Arrays.asList();
        }

        List<String> documentUniqueIds = extractDocumentUniqueIds(deletedDocumentEntries);
        List<String> existingDocumentUniqueIds = openXdsDao.findExistingDocumentUniqueIds(documentUniqueIds);

        openXdsDao.deleteDocuments(existingDocumentUniqueIds);

        return getDocumentsEntriesDeleted(deletedDocumentEntries, existingDocumentUniqueIds);
    }

    private List<String> extractDocumentUniqueIds(List<DocumentEntry> deletedDocumentEntries) {
        return deletedDocumentEntries.stream().map(DocumentEntry::getUniqueId).collect(Collectors.toList());
    }

    private List<DocumentEntry> getDocumentsEntriesDeleted(List<DocumentEntry> deletedDocumentEntries, List<String> existingDocumentUniqueIds) {
        return deletedDocumentEntries.stream().filter(dde -> existingDocumentUniqueIds.contains(dde.getUniqueId())).collect(Collectors.toList());
    }
}
