package dk.nsp.xdscleanup.openxds.health;

import dk.nsp.backgroundjob.health.HealthIndication;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;
import org.springframework.jdbc.BadSqlGrammarException;
import org.springframework.jdbc.core.JdbcTemplate;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

@RunWith(MockitoJUnitRunner.class)
public class OpenXdsHealthIndicatorTest {
    @InjectMocks
    private OpenXdsHealthIndicator subject;

    @Mock
    private JdbcTemplate template;

    @Test
    public void getHealthInfo_databaseIsOk() {
        // Arrange
        Mockito.when(template.queryForObject(Mockito.anyString(), Mockito.any(Class.class)))
                .thenReturn(1);

        // Act
        HealthIndication result = subject.getHealthInfo();

        // Assert
        assertEquals(200, result.getStatus());
        assertTrue(result.getText().contains("OK"));
    }

    @Test
    public void getHealthInfo_databaseIsDead() {
        // Arrange
        Mockito.when(template.queryForObject(Mockito.anyString(), Mockito.any(Class.class)))
                .thenThrow(BadSqlGrammarException.class);

        // Act
        HealthIndication result = subject.getHealthInfo();

        // Assert
        assertEquals(500, result.getStatus());
        assertTrue(result.getText().contains("ERROR"));
    }
}