package dk.nsp.xdscleanup.openxds.service;

import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.openxds.dao.OpenXdsDao;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Arrays;
import java.util.List;

import static dk.nsp.xdscleanup.openxds.TestConstants.*;
import static org.junit.Assert.assertEquals;

@RunWith(MockitoJUnitRunner.class)
public class RepositoryDeletionServiceTest {
    @InjectMocks
    private RepositoryDeletionService subject;

    @Mock
    private OpenXdsDao openXdsDao;

    private static final DocumentEntry DOCUMENT_ENTRY_0 = new DocumentEntry(DOCUMENT_ENTRY_ID_0, DOCUMENT_ENTRY_UUID_0, DOCUMENT_UNIQUE_ID_0);
    private static final DocumentEntry DOCUMENT_ENTRY_1 = new DocumentEntry(DOCUMENT_ENTRY_ID_1, DOCUMENT_ENTRY_UUID_1, DOCUMENT_UNIQUE_ID_1);
    private static final DocumentEntry DOCUMENT_ENTRY_2 = new DocumentEntry(DOCUMENT_ENTRY_ID_2, DOCUMENT_ENTRY_UUID_2, DOCUMENT_UNIQUE_ID_2);
    private static final DocumentEntry DOCUMENT_ENTRY_3 = new DocumentEntry(DOCUMENT_ENTRY_ID_3, DOCUMENT_ENTRY_UUID_3, DOCUMENT_UNIQUE_ID_3);

    @Test
    public void deleteDocuments_emptyList_noop() {
        // Arrange
        List<DocumentEntry> documentEntries = Arrays.asList();

        // Act
        List<DocumentEntry> result = subject.deleteDocuments(documentEntries);

        // Assert
        assertEquals(0, result.size());
        Mockito.verify(openXdsDao, Mockito.never()).deleteDocuments(Mockito.anyList());
    }

    @Test
    public void deleteDocuments_resultIncludesDeletedDocuments() {
        // Arrange
        List<DocumentEntry> documentEntries = Arrays.asList(DOCUMENT_ENTRY_0, DOCUMENT_ENTRY_1);

        Mockito.when(openXdsDao.findExistingDocumentUniqueIds(Arrays.asList(DOCUMENT_UNIQUE_ID_0, DOCUMENT_UNIQUE_ID_1)))
                .thenReturn(Arrays.asList(DOCUMENT_UNIQUE_ID_0, DOCUMENT_UNIQUE_ID_1));

        // Act
        List<DocumentEntry> result = subject.deleteDocuments(documentEntries);

        // Assert
        assertEquals(2, result.size());
        assertEquals(DOCUMENT_UNIQUE_ID_0, result.get(0).getUniqueId());
        assertEquals(DOCUMENT_UNIQUE_ID_1, result.get(1).getUniqueId());
    }

    @Test
    public void deleteDocuments_resultExcludesMissingDocuments() {
        // Arrange
        List<DocumentEntry> documentEntries = Arrays.asList(DOCUMENT_ENTRY_0, DOCUMENT_ENTRY_1, DOCUMENT_ENTRY_2, DOCUMENT_ENTRY_3);

        Mockito.when(openXdsDao.findExistingDocumentUniqueIds(Arrays.asList(DOCUMENT_UNIQUE_ID_0, DOCUMENT_UNIQUE_ID_1, DOCUMENT_UNIQUE_ID_2, DOCUMENT_UNIQUE_ID_3)))
                .thenReturn(Arrays.asList(DOCUMENT_UNIQUE_ID_2, DOCUMENT_UNIQUE_ID_3));

        // Act
        List<DocumentEntry> result = subject.deleteDocuments(documentEntries);

        // Assert
        assertEquals(2, result.size());
        assertEquals(DOCUMENT_UNIQUE_ID_2, result.get(0).getUniqueId());
        assertEquals(DOCUMENT_UNIQUE_ID_3, result.get(1).getUniqueId());
    }

    @Test
    public void deleteDocuments_documentsAreDeleted() {
        // Arrange
        List<DocumentEntry> documentEntries = Arrays.asList(DOCUMENT_ENTRY_0, DOCUMENT_ENTRY_1);

        List<String> uniqueIds = Arrays.asList(DOCUMENT_UNIQUE_ID_0, DOCUMENT_UNIQUE_ID_1);
        Mockito.when(openXdsDao.findExistingDocumentUniqueIds(uniqueIds))
                .thenReturn(uniqueIds);

        // Act
        List<DocumentEntry> result = subject.deleteDocuments(documentEntries);

        // Assert
        Mockito.verify(openXdsDao).deleteDocuments(uniqueIds);
    }
}